//   Connections between ADC and Arduino:
// 
//  |   MPU6050 pins    |        Pin function       | Arduino pins     |
//  |------------------------------------------------------------------| 
//  | VCC               | power supply (+)          | 5V               | Supply and Ground
//  | GND               | Ground                    | GND              |      
//  |------------------------------------------------------------------|
//  | INT               | Data ready                | 2 Interrupt Pin  | 
//  | SDA               | Data line                 | SDA (A4 - D2     | I2C communication
//  | SCL               | Serial clock              | SCL (A5 - D1     |
//  | AD0               | Address bit #0  (1,0)     | GND or 5V        |
//  |------------------------------------------------------------------|

#include <Wire.h>
#include <ESP8266WiFi.h>

// Configurazione Wi-Fi
const char* ssid = "PoliMI-Full";      // Nome della tua rete Wi-Fi
const char* password = "wificarlo";           // Password della tua rete Wi-Fi
const char* serverIP = "192.168.152.105";       // Indirizzo IP del server a cui inviare i dati
const uint16_t serverPort = 8080;             // Porta del server
//per configurare la usb su Linux: 

WiFiClient client;


// MPU6050 I2C address 
const int MPU_Addr_GND = 0x68;    //if AD0=GND: 1101000
const int MPU_Addr_5V = 0x69;     //if AD0=5V:  1101001

const int ACC_CONFIG_VALUE = 0x10; //0001 0000 -> 0x10
const int PWR_MGMT_2 = 0x07; //We read the x,y,z acc 0000 0111-> 0x07

//POWER MANAGEMENT 1
const int RESET_DEVICE = 0x80; //1000 0000 ->0x80
const int SLEEP = 0x48; //0100 1000 ->  0x48
const int NORMAL = 0x08; //0000 1000 ->0x8

const int SR_DIV = 7; //IMPOSTO FREQUENZA DI ACQUISIZIONE (79  = 100 Hz (8000/(79+1))

const int ACC_X_ID = 0x3B;
const int ACC_Y_ID = 0x3D;
const int ACC_Z_ID = 0x3F; //check from the datasheet register 59-64, we read z axis so 3F and 40


// Variabili per la media dei primi 2 secondi
float sum_ax = 0.0, sum_ay = 0.0, sum_az = 0.0;
unsigned long start_time = 0;
unsigned int samples_count = 0;


void setup() {
  Wire.begin();
  Serial.begin(115200);

  sensorConfiguration();
  wifiConnection();

  // Calcolo dell'offset nei primi 2 secondi
  start_time = millis();
  while (millis() - start_time < 2000) {
    sum_ax += read_acc(MPU_Addr_GND, ACC_X_ID);
    sum_ay += read_acc(MPU_Addr_GND, ACC_Y_ID);
    sum_az += read_acc(MPU_Addr_GND, ACC_Z_ID);
    samples_count++;
    delay(10);
  }

  // Calcola la media
  sum_ax /= samples_count;
  sum_ay /= samples_count;
  sum_az /= samples_count;
}


float timestamp = 0.0;
void loop() {
int numDati = 4;
  float acc[numDati];
  String accData;
  int i = 0;

  timestamp = timestamp + 0.01;
  acc[0] = timestamp;
  acc[1] = read_acc(MPU_Addr_GND, ACC_X_ID) - sum_ax;
  acc[2] = read_acc(MPU_Addr_GND, ACC_Y_ID) - sum_ay;
  acc[3] = read_acc(MPU_Addr_GND, ACC_Z_ID) - sum_az;

  for (i = 0; i < numDati; i++) {
    if (i > 0) {
      accData += ",";
    }
    accData += String(acc[i], 4); // Formatta con 4 decimali
  }


  // Invio dei dati tramite Wi-Fi
  if (client.connected()) {
    Serial.println("Data:");
    client.println("Data:");
    //delay(2);
    Serial.println(accData);
    client.println(accData);
  } else {
    Serial.println("Client disconnesso. Tentativo di riconnessione...");
    if (client.connect(serverIP, serverPort)) {
      Serial.println("Riconnessione al server riuscita");
    } else {
      Serial.println("Riconnessione al server fallita");
    }
  }

  
  delay(7); //is around 100hz
}


float read_acc(int deviceAddress, int accID){
  byte value1, value2;
  Wire.beginTransmission(deviceAddress);
  Wire.write(accID);
  Wire.endTransmission(false);
  Wire.requestFrom(deviceAddress,2);
  value1 = Wire.read();
  value2 = Wire.read();
  
  int16_t full_value = (value1 << 8) | value2;
  Wire.endTransmission(true);

  return full_value*2.0*8.0*9.81/ pow(2,16); //becouse I set +- 8g as FS(fullscale);
}


//INITIALIZATION FUNCTIONS

void sensorConfiguration() {
  //Let's read and print the registerValue before and after sending two different commands
  writeRegisterValue(MPU_Addr_GND,0x6B, RESET_DEVICE);
  delay(1000);
  byte device_status = readRegisterValue(MPU_Addr_GND,0x6B);

  Serial.print("before: ");
  Serial.println(device_status, HEX);

  writeRegisterValue(MPU_Addr_GND,0x6B, NORMAL);
  delay(1000);
  device_status = readRegisterValue(MPU_Addr_GND,0x6B);

  Serial.print("after : ");
  Serial.println(device_status, HEX);

  writeRegisterValue(MPU_Addr_GND,0x6C,PWR_MGMT_2);
  writeRegisterValue(MPU_Addr_GND,0x1C,ACC_CONFIG_VALUE);
  writeRegisterValue(MPU_Addr_GND,0x19, SR_DIV);
}

byte readRegisterValue(int deviceAddress, byte registerAddress){
  byte registerValue; 
  Wire.beginTransmission(deviceAddress);
  Wire.write(registerAddress);
  Wire.endTransmission(false);
  Wire.requestFrom(deviceAddress,1);
  registerValue = Wire.read();
  Wire.endTransmission(true);
  return registerValue;
}

void writeRegisterValue(int deviceAddress, byte registerAddress, byte registerValue){ //this function write the register value on the memory 
  Wire.beginTransmission(deviceAddress);
  Wire.write(registerAddress);
  Wire.write(registerValue);
  Wire.endTransmission(true);
  delay(3);
}


void wifiConnection() {
  // Connessione alla rete Wi-Fi
  Serial.print("Connessione a ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  
  Serial.println("");
  Serial.println("WiFi connesso");
  Serial.print("IP Address: ");
  Serial.println(WiFi.localIP());

  // Tentativo di connessione al server
  if (client.connect(serverIP, serverPort)) {
    Serial.println("Connessione al server riuscita");
  } else {
    Serial.println("Connessione al server fallita");
  }
}
